package net.sf.sql2java.data;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class Path {
	public static final String SEPARATOR = "/";
	private List<Pointer> elements;
	
// -- Constructors ------------------------------------------------------------
	
	public Path() {
		this(new ArrayList<Pointer>());
	}
	
	public Path(List<Pointer> elements) {
		this.elements = elements;
	}
	
// -- Convertors --------------------------------------------------------------
	
	public String toString() {
		return toString(SEPARATOR);
	}

	public String toString(String separator) {
		StringBuffer result = new StringBuffer();
		
		for (Pointer element : elements) {
			if (elements.indexOf(element) != 0)
				result.append(separator);
			
			result.append(element.toString());
		}
		
		return result.toString();
	}
	
// -- Factory Method ----------------------------------------------------------
	
	public static Path createPath(String path) {		
		return createPath(path, SEPARATOR);
	}
	
	public static Path createPath(String path, String separator) {
		// -- create the result
		Path result = new Path();
		
		StringTokenizer tokenizer = new StringTokenizer(path, separator);
		while (tokenizer.hasMoreTokens())
			result.append(tokenizer.nextToken());
		
		// -- return the result
		return result;
	}
	
// -- Element Manipulation ----------------------------------------------------
	
	public Pointer getElement(String name) {
		for (Pointer element : elements)
			if (element.equals(name))
				return element;
		
		return null;
	}
	
	public Pointer getLastElement() {
		if (elements.size() == 0)
			return null;
		else
			return elements.get(elements.size() - 1);
	}
	
	public Pointer getFirstElement() {
		if (elements.size() == 0)
			return null;
		else
			return elements.get(0);
	}
	
	public Pointer getElement(int index) {
		return elements.get(index);
	}
	
	public List<Pointer> getElements() {
		return elements;
	}

// -- Searching ---------------------------------------------------------------
	
	public Path getSubPath(String key) {
		Pointer p = getElement(key);
		
		// -- get the index of the element
		int index = elements.indexOf(p);
		
		// -- return the subpath based on the index
		return getSubPath(index);
	}
	
	public Path getSubPath(Pointer element) {
		// -- get the index of the element
		int index = elements.indexOf(element);
		
		// -- return the subpath based on the index
		return getSubPath(index);
	}
	
	public Path getSubPath(int index) {
		return getSubPath(index, this.elements.size());
	}
	
	public Path getSubPath(int startIndex, int endIndex) {
		// -- check if the index is invalid, in which case we'll return 'null'
		if (startIndex > endIndex) return null;
		if (startIndex < 0) return null;
		if (endIndex > this.elements.size()) return null;
		
		// -- create a sublist starting at the given index till the end
		List<Pointer> elements = this.elements.subList(startIndex, endIndex);
		
		// -- create a path from the sublist and return it
		return new Path(elements);
	}
	
// -- Appending ---------------------------------------------------------------
	
	public Path append(String element) {
		if (element.indexOf(MapPointer.START_SEPARATOR) == -1)
			return append(new Pointer(element));
		else
			return append(MapPointer.createPointer(element));
	}
	
	public Path append(Pointer element) {
		elements.add(element);
		
		return this;
	}
	
// -- Properties --------------------------------------------------------------
	
	public int getLength() {
		return this.elements.size();
	}
}
